@@ -8,15 +8,14 @@ import json
 import logging
 import time
 from contextlib import asynccontextmanager
-from typing import AsyncGenerator
+from typing import AsyncGenerator, Optional
 import asyncio
-from sqlalchemy import Column, event, Table, ForeignKey, text, AsyncAdaptedQueuePool
+from sqlalchemy import Column, event, Table, ForeignKey, text, AsyncAdaptedQueuePool, VARCHAR
 from sqlalchemy.dialects.postgresql import CITEXT, JSONB
 from sqlalchemy.engine import Engine
 from sqlalchemy.ext.asyncio import create_async_engine, AsyncSession, async_sessionmaker
 from sqlalchemy.orm import declarative_base, relationship
 from sqlalchemy.types import TypeDecorator, TEXT, String, Text, Boolean, Integer, Float
-
 from authlib.integrations.sqla_oauth2 import OAuth2ClientMixin
 from authtuna.core.config import settings
 from authtuna.core.encryption import encryption_utils
@@ -118,7 +117,9 @@ user_roles_association = Table(
     Column('user_id', String(64), ForeignKey('users.id'), primary_key=True),
     Column('role_id', Integer, ForeignKey('roles.id'), primary_key=True),
     Column('scope', String(255), primary_key=True, default='none', nullable=False),
-    Column('given_by_id', String(64), ForeignKey('users.id'), nullable=False),
+    Column('given_by_id', VARCHAR(64),
+           # ForeignKey('users.id'),
+           nullable=False),
     Column('given_at', Float, nullable=False, default=time.time),
 )
 
@@ -131,6 +132,20 @@ role_permissions_association = Table(
 )
 
 
+role_assign_permissions = Table(
+    'role_assign_permissions', Base.metadata,
+    Column('assigner_role_id', Integer, ForeignKey('roles.id'), primary_key=True),
+    Column('assignable_role_id', Integer, ForeignKey('roles.id'), primary_key=True)
+)
+
+
+role_grant_permissions = Table(
+    'role_grant_permissions', Base.metadata,
+    Column('granter_role_id', Integer, ForeignKey('roles.id'), primary_key=True),
+    Column('grantable_permission_id', Integer, ForeignKey('permissions.id'), primary_key=True)
+)
+
+
 # --- ORM Models ---
 
 class User(Base):
@@ -145,7 +160,7 @@ class User(Base):
                       unique=True, nullable=False, index=True)
     email = Column(CaseInsensitiveText(120, collation='NOCASE' if engine.dialect.name == 'sqlite' else None),
                    unique=True, nullable=False, index=True)
-
+    is_active = Column(Boolean, default=True, nullable=False)
     password_hash = Column(String(256), nullable=True)
     email_verified = Column(Boolean, default=False, nullable=False)
     requires_password_reset = Column(Boolean, default=False, nullable=False)
@@ -179,7 +194,7 @@ class User(Base):
             {"had_old_password": bool(old_hash)}, db=db,
         )
 
-    async def check_password(self, password: str, ip: str, db_manager_custom=None, db: AsyncSession = None) -> bool | None:
+    async def check_password(self, password: str, ip: str, db_manager_custom=None, db: AsyncSession = None) -> Optional[bool]:
         """
         Asynchronously checks the password and logs login attempts.
         Returns True if valid, False otherwise.
@@ -223,6 +238,8 @@ class Role(Base):
     name = Column(String(80), unique=True, nullable=False)
     description = Column(String(255))
     system = Column(Boolean, default=False, nullable=False)
+    level = Column(Integer, nullable=True)
+
     users = relationship(
         "User", secondary=user_roles_association, back_populates="roles",
         primaryjoin=lambda: Role.id == user_roles_association.c.role_id,
@@ -230,6 +247,23 @@ class Role(Base):
     )
     permissions = relationship("Permission", secondary=role_permissions_association, back_populates="roles", lazy="joined")
 
+    can_assign_roles = relationship(
+        "Role",
+        secondary=role_assign_permissions,
+        primaryjoin=id == role_assign_permissions.c.assigner_role_id,
+        secondaryjoin=id == role_assign_permissions.c.assignable_role_id,
+        backref="assignable_by_roles",
+        lazy="joined"
+    )
+
+    can_grant_permissions = relationship(
+        "Permission",
+        secondary=role_grant_permissions,
+        primaryjoin=id == role_grant_permissions.c.granter_role_id,
+        secondaryjoin=lambda: Permission.id == role_grant_permissions.c.grantable_permission_id,
+        backref="grantable_by_roles",
+        lazy="joined"
+    )
     def __repr__(self):
         return f"<Role {self.name}>"
 
@@ -267,6 +301,7 @@ class Session(Base):
     last_ip = Column(String(45))
     user = relationship('User', back_populates='sessions')
     random_string = Column(String(255), nullable=False, default=encryption_utils.gen_random_string, onupdate=encryption_utils.gen_random_string)
+    previous_random_strings = Column(JsonType, nullable=False, default=list)
 
     def is_expired(self):
         return time.time() > self.etime or time.time() > self.e_abs_time
@@ -285,11 +320,16 @@ class Session(Base):
                 await db_manager.log_audit_event(self.user_id, "SESSION_INVALIDATED", self.last_ip, {"reason": "device_mismatch"}, db=db)
                 await self.terminate(self.last_ip, db=db)
                 return False
-            if self.random_string == random_string:
+            is_token_valid = (self.random_string == random_string or
+                              any(entry['value'] == random_string for entry in self.previous_random_strings))
+
+            if is_token_valid:
                 return True
             else:
-                await db_manager.log_audit_event(self.user_id, "SESSION_INVALIDATED", self.last_ip, {"reason": "random_string_mismatch"}, db=db)
+                await db_manager.log_audit_event(self.user_id, "SESSION_INVALIDATED", self.last_ip,
+                                                 {"reason": "random_string_mismatch"}, db=db)
                 await self.terminate(self.last_ip, db=db)
+                return False
         return False
 
     async def update_last_ip(self, ip: str, db_manager_custom=None, db: AsyncSession = None):
@@ -319,8 +359,17 @@ class Session(Base):
         """
         Rotates the per-request random_string to mitigate replay attacks.
         """
+        now = time.time()
+
+        new_history = [{'value': self.random_string, 'timestamp': now}]
+
+        grace_period = settings.SESSION_DB_VERIFICATION_INTERVAL + 5
+        for entry in self.previous_random_strings:
+            if now - entry.get('timestamp', 0) < grace_period:
+                new_history.append(entry)
+        self.previous_random_strings = new_history
         self.random_string = encryption_utils.gen_random_string()
-        self.mtime = time.time()
+        self.mtime = now
         return self.random_string
 
     async def terminate(self, ip: str, db_manager_custom=None, db: AsyncSession = None):
@@ -439,6 +488,9 @@ class DatabaseManager:
 
         async with engine.begin() as conn:
             await conn.run_sync(Base.metadata.create_all)
+        from authtuna.core.defaults import provision_defaults
+        async with self.AsyncSessionLocal() as session:
+            await provision_defaults(session)
 
     @asynccontextmanager
     async def get_context_manager_db(self) -> AsyncGenerator[AsyncSession, None]:
